searchNodes=[{"ref":"StatBuffer.html","title":"StatBuffer","type":"behaviour","doc":"Defines a stat buffer. A stat buffer is an efficient way to maintain a local incrementable count with a given key that can later be flushed to persistent storage. In fast moving systems, this provides a scalable way keep track of counts without putting heavy loads on a database. Creating a buffer is as easy as: defmodule Buffer do use StatBuffer end Once we have defined our buffer module, all we must do is implement a handle_flush/2 callback that allows us to perform an operation with a provided key and counter. This typically means updating a counter in a database. defmodule Buffer do use StatBuffer def handle_flush(key, counter) do # write to the database... # handle_flush MUST return an :ok atom :ok end end Each flush operation is handled with its own supervised Task process. By default, a failed flush operation will retry about 3 times within 3 seconds. Usage Before using our buffer, we must start it. Buffer.start() With our buffer started, we can now increment key counters. A key can be any valid term. Buffer.increment(&quot;mykey&quot;) # increments by 1 Buffer.increment(&quot;mykey&quot;, 10) # increments by 10 Buffer.async_increment(&quot;mykey&quot;) # async increments by 1 Key counts are maintained in an ETS table. All keys are scoped to the given buffer module - so multiple buffers using the same keys will not cause issues. With the default buffer we setup above, the &quot;mykey&quot; counter will be flushed after 5 seconds. Assuming no new operations occur with our buffer, the process will be placed into hibernation after 10 seconds. All of this is configurable through the options below. Options A stat buffer comes with a few configurable options. We can pass any of these options along with the use macro. use StatBuffer, interval: 60_000, jitter: 20_000 :interval - the time in milliseconds between the first increment for a given key and its next flush callback being invoked. Defaults to 5_000. :jitter - a max time in milliseconds that will be added to interval to ensure some randomness in each flush invocation. The time added would be randomly selected between 0 and jitter. Defaults to 0. :timeout - the time in milliseconds between the last operation on a a buffer, and the process being hibernated. Defaults to 10_000. :backoff - the time in milliseconds between a handle_flush/2 callback failing, and the next attempt occuring. Defaults to 1_000. :restart - the :restart option used for the flush task. Please see Task.Supervisor.start_child/2 for more details. :shutdown - the :shutdown option used for the flush task. Please see Task.Supervisor.start_child/2 for more details. Autostart We can start our buffers automatically by adding them to the application config. config :stat_buffer, buffers: [ MyBufferOne MyBufferTwo ] Now, when the stat_buffer application starts - your buffers will also be started."},{"ref":"StatBuffer.html#c:async_increment/2","title":"StatBuffer.async_increment/2","type":"callback","doc":"Same as increment/2 except performs the operation asynchronously."},{"ref":"StatBuffer.html#c:backoff/0","title":"StatBuffer.backoff/0","type":"callback","doc":"The amount of time that will be applied between failed key flush attempts."},{"ref":"StatBuffer.html#c:count/1","title":"StatBuffer.count/1","type":"callback","doc":"Returns the current state of a key from the buffer."},{"ref":"StatBuffer.html#c:flush/1","title":"StatBuffer.flush/1","type":"callback","doc":"Asynchronously flushes a given key from the buffer."},{"ref":"StatBuffer.html#c:handle_flush/2","title":"StatBuffer.handle_flush/2","type":"callback","doc":"Callback for flushing a key for the buffer. When a buffer key hits its set time interval, this function will be called and provided with the key as well its current counter. This function is called within its own Task and is supervised. If the callback does not return :ok - the task will fail and attempt a retry with configurable backoff. Parameters key: Any valid term. counter: An integer counter."},{"ref":"StatBuffer.html#c:increment/2","title":"StatBuffer.increment/2","type":"callback","doc":"Increments a given key in the buffer by the provided count. Each key is scoped to the buffer module. So duplicate keys across different buffer modules will not cause issues. Parameters key: Any valid term. count: An integer count. Defaults to 1."},{"ref":"StatBuffer.html#c:interval/0","title":"StatBuffer.interval/0","type":"callback","doc":"The amount of time between buffer flush operations. If specified in the options, a jitter may be applied."},{"ref":"StatBuffer.html#reset/0","title":"StatBuffer.reset/0","type":"function","doc":""},{"ref":"StatBuffer.html#c:start/0","title":"StatBuffer.start/0","type":"callback","doc":"Starts the buffer process."},{"ref":"StatBuffer.html#c:task_opts/0","title":"StatBuffer.task_opts/0","type":"callback","doc":"The options that will be used when a supervised Task is started to flush the buffer."},{"ref":"StatBuffer.html#c:timeout/0","title":"StatBuffer.timeout/0","type":"callback","doc":"The amount of time that a buffer key process will remain without recieving work before shutting down."},{"ref":"StatBuffer.html#t:t/0","title":"StatBuffer.t/0","type":"type","doc":""},{"ref":"StatBuffer.Error.html","title":"StatBuffer.Error","type":"exception","doc":""},{"ref":"readme.html","title":"StatBuffer","type":"extras","doc":"StatBuffer StatBuffer is an efficient way to maintain a local incrementable count with a given key that can later be flushed to persistent storage. In fast moving systems, this provides a scalable way keep track of counts without putting heavy loads on a database."},{"ref":"readme.html#installation","title":"StatBuffer - Installation","type":"extras","doc":"The package can be installed by adding stat_buffer to your list of dependencies in mix.exs: def deps do [ {:stat_buffer, &quot;~&gt; 0.3.0&quot;} ] end"},{"ref":"readme.html#documentation","title":"StatBuffer - Documentation","type":"extras","doc":"Please see HexDocs for additional documentation. This readme provides a brief overview, but it is recommended that the docs are used."},{"ref":"readme.html#creating-a-buffer","title":"StatBuffer - Creating a Buffer","type":"extras","doc":"We can start off by creating our buffer. This is simply a module that uses StatBuffer and implements the handle_flush/2 callback. defmodule Buffer do use StatBuffer def handle_flush(key, counter) do # do database stuff... # we must return an :ok atom :ok end end We must now start our buffer process. Buffer.start() There are some configruable options available for our buffers. You can read more about them here. These options can be passed when creating our buffer. use StatBuffer, interval: 10_000 With our buffer started, we can now increment key counters. A key can be any valid term. Buffer.increment(&quot;mykey&quot;) # increments by 1 Buffer.increment(&quot;mykey&quot;, 10) # increments by 10 Buffer.async_increment(&quot;mykey&quot;) # async increments by 1 And we&#39;re done! Our counter will be flushed using our handle_flush/2 callback after the default interval period. Dead counters are automatically removed."},{"ref":"readme.html#buffer-autostart","title":"StatBuffer - Buffer Autostart","type":"extras","doc":"We can also start our buffers automatically by adding them to the application config. config :stat_buffer, buffers: [ MyBufferOne MyBufferTwo ] Now, when the stat_buffer application starts - your buffers will also be started."}]